@@ -18,6 +18,7 @@ import os
 import logging
 from gi.repository import Gtk
 from gi.repository import GObject
+from gi.repository import GLib
 from gi.repository import GdkPixbuf
 
 import zim.fs
@@ -162,7 +163,7 @@ def get_mime_icon(file, size):
 					logger.debug('Missing icons in icon theme: %s', names)
 					_last_warning_missing_icon = names
 				return None
-		except GObject.GError:
+		except (GObject.GError, GLib.Error):
 			logger.exception('Could not load icon for file: %s', file)
 			return None
 	else:
@@ -198,7 +199,7 @@ def _read_comment_from(file):
 			lang = elt.attrib.get(xmlns + 'lang', '')
 			if lang == mylang:
 				return elt.text
-			elif not lang or mylang.startswith(lang + '_'):
+			elif not lang or mylang and mylang.startswith(lang + '_'):
 				fallback.append((lang, elt.text))
 			else:
 				pass
