@@ -2,8 +2,10 @@
 # Copyright 2008-2018 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
 import os
+import sys
 import logging
 from gi.repository import GObject
+from gi.repository import GLib
 from gi.repository import Gtk
 from gi.repository import Gdk
 
@@ -15,6 +17,7 @@ from zim.signals import DelayedCallback
 
 from zim.notebook import Path, Page, LINK_DIR_BACKWARD
 from zim.notebook.index import IndexNotFoundError
+from zim.notebook.operations import ongoing_operation
 from zim.history import History, HistoryPath
 
 from zim.actions import action, toggle_action, radio_action, radio_option, get_gtk_actiongroup, \
@@ -232,10 +235,11 @@ class MainWindow(Window):
 		self.statusbar.push(0, '<page>')
 		self.add_bar(self.statusbar, start=False)
 		self.statusbar.set_property('margin', 0)
+		self.statusbar.set_property('spacing', 0)
 
 		def statusbar_element(string, size):
 			frame = Gtk.Frame()
-			frame.set_shadow_type(Gtk.ShadowType.IN)
+			frame.set_shadow_type(Gtk.ShadowType.NONE)
 			self.statusbar.pack_end(frame, False, True, 0)
 			label = Gtk.Label(label=string)
 			label.set_size_request(size, 10)
@@ -244,15 +248,17 @@ class MainWindow(Window):
 			return label
 
 		# specify statusbar elements right-to-left
-		self.statusbar_style_label = statusbar_element('<style>', 100)
 		self.statusbar_insert_label = statusbar_element('INS', 60)
+		self.statusbar_style_label = statusbar_element('<style>', 110)
 
 		# and build the widget for backlinks
 		self.statusbar_backlinks_button = \
 			BackLinksMenuButton(self.notebook, self.open_page, status_bar_style=True)
 		frame = Gtk.Frame()
-		frame.set_shadow_type(Gtk.ShadowType.IN)
+		frame.set_shadow_type(Gtk.ShadowType.NONE)
+		self.statusbar.pack_end(Gtk.Separator(), False, True, 0)
 		self.statusbar.pack_end(frame, False, True, 0)
+		self.statusbar.pack_end(Gtk.Separator(), False, True, 0)
 		frame.add(self.statusbar_backlinks_button)
 
 		self.move_bottom_minimized_tabs_to_statusbar(self.statusbar)
@@ -283,6 +289,10 @@ class MainWindow(Window):
 		self.uimanager.insert_action_group(group, 0)
 
 		group = get_gtk_actiongroup(self)
+		# don't use mnemonics on macOS to allow alt-<letter> shortcuts
+		global MENU_ACTIONS
+		if sys.platform == "darwin":
+			MENU_ACTIONS = tuple((t[0], t[1], t[2].replace('_', '')) for t in MENU_ACTIONS)
 		group.add_actions(MENU_ACTIONS)
 		self.uimanager.insert_action_group(group, 0)
 
@@ -338,25 +348,31 @@ class MainWindow(Window):
 		closing if there are no other toplevel windows.
 		'''
 		if self.hideonclose: # XXX
-			self.save_uistate()
-			self.hide()
-			self.emit('close')
+			self._do_close()
 		else:
 			self.destroy()
 
+	def _do_close(self):
+		self.save_uistate()
+		self.hide()
+		self.emit('close')
+
 	def destroy(self):
 		self.pageview.save_changes()
 		if self.page.modified:
 			return # Do not quit if page not saved
 		self.pageview.page.set_ui_object(None) # XXX
 
-		self.save_uistate()
+		self._do_close()
 
-		self.hide() # look more responsive
-		self.notebook.index.stop_background_check()
 		while Gtk.events_pending():
 			Gtk.main_iteration_do(False)
 
+		self.notebook.index.stop_background_check()
+		op = ongoing_operation(self.notebook)
+		if op:
+			op.wait()
+
 		Window.destroy(self) # gtk destroy & will also emit destroy signal
 
 	def do_update_statusbar(self, *a):
@@ -395,7 +411,7 @@ class MainWindow(Window):
 			# Hidden param, disabled because it causes problems with
 			# several international layouts (space mistaken for alt-space,
 			# see bug lp:620315)
-			group.connect_group( # <Alt><Space>
+			group.connect( # <Alt><Space>
 				space, Gdk.ModifierType.MOD1_MASK, Gtk.AccelFlags.VISIBLE,
 				self.toggle_sidepane_focus)
 
@@ -667,6 +683,7 @@ class MainWindow(Window):
 		self.toolbar.insert(item, -1)
 
 		# Load accelmap config and setup saving it
+		# TODO - this probably belongs in the application class, not here
 		accelmap = ConfigManager.get_config_file('accelmap').file
 		logger.debug('Accelmap: %s', accelmap.path)
 		if accelmap.exists():
@@ -696,7 +713,7 @@ class MainWindow(Window):
 		if self.notebook.icon:
 			try:
 				self.set_icon_from_file(self.notebook.icon)
-			except GObject.GError:
+			except (GObject.GError, GLib.Error):
 				logger.exception('Could not load icon %s', self.notebook.icon)
 
 	def on_textview_toggle_overwrite(self, view):
@@ -707,8 +724,8 @@ class MainWindow(Window):
 			text = 'INS'
 		self.statusbar_insert_label.set_text(text)
 
-	def on_textview_textstyle_changed(self, view, style):
-		label = style.title() if style else 'None'
+	def on_textview_textstyle_changed(self, view, styles):
+		label = ", ".join([s.title() for s in styles if s]) if styles else 'None'
 		self.statusbar_style_label.set_text(label)
 
 	def on_link_enter(self, view, link):
@@ -732,9 +749,14 @@ class MainWindow(Window):
 		'''Method to open a page in the mainwindow, and menu action for
 		the "jump to" menu item.
 
+		Fails silently when saving current page failed (which is usually the
+		result of pressing "cancel" in the error dialog shown when saving
+		fails). Check return value for success if you want to be sure.
+
 		@param path: a L{path} for the page to open.
 		@raises PageNotFound: if C{path} can not be opened
 		@emits: page-changed
+		@returns: C{True} for success
 		'''
 		assert isinstance(path, Path)
 		if isinstance(path, Page) and path.valid:
@@ -750,7 +772,7 @@ class MainWindow(Window):
 			self.pageview.save_changes() # XXX - should connect to signal instead of call here
 			self.notebook.wait_for_store_page_async() # XXX - should not be needed - hide in notebook/page class - how?
 			if self.page.modified:
-				raise AssertionError('Could not save page') # XXX - shouldn't this lead to dialog ?
+				return False # Assume SavePageErrorDialog was shown and cancelled
 
 			old_cursor = self.pageview.get_cursor_pos()
 			old_scroll = self.pageview.get_scroll_pos()
@@ -820,8 +842,8 @@ class MainWindow(Window):
 
 	@action(
 		_('_Back'), verb_icon='gtk-go-back', # T: Menu item
-		accelerator='<alt>Left', alt_accelerator=('XF86Back' if os.name != 'nt' else None)
-	)	# The XF86 keys are mapped wrongly on windows, see bug lp:1277929
+		accelerator='<alt>Left', alt_accelerator='XF86Back'
+	)
 	def open_page_back(self):
 		'''Menu action to open the previous page from the history
 		@returns: C{True} if successfull
@@ -832,8 +854,8 @@ class MainWindow(Window):
 
 	@action(
 		_('_Forward'), verb_icon='gtk-go-forward', # T: Menu item
-		accelerator='<alt>Right', alt_accelerator=('XF86Forward' if os.name != 'nt' else None)
-	)	# The XF86 keys are mapped wrongly on windows, see bug lp:1277929
+		accelerator='<alt>Right', alt_accelerator='XF86Forward'
+	)
 	def open_page_forward(self):
 		'''Menu action to open the next page from the history
 		@returns: C{True} if successfull
@@ -899,8 +921,8 @@ class MainWindow(Window):
 		pos = self.pageview.get_cursor_pos()
 		self.pageview.save_changes() # XXX
 		self.notebook.flush_page_cache(self.page)
-		self.open_page(self.notebook.get_page(self.page))
-		self.pageview.set_cursor_pos(pos)
+		if self.open_page(self.notebook.get_page(self.page)):
+			self.pageview.set_cursor_pos(pos)
 
 
 class BackLinksMenuButton(MenuButton):
@@ -919,7 +941,7 @@ class BackLinksMenuButton(MenuButton):
 			n = 0
 
 		self.label.set_text_with_mnemonic(
-			ngettext('%i _Backlink...', '%i _Backlinks...', n) % n)
+			ngettext('%i _Backlink', '%i _Backlinks', n) % n)
 			# T: Label for button with backlinks in statusbar
 		self.set_sensitive(n > 0)
 
@@ -952,7 +974,7 @@ class PageWindow(Window):
 		#if ui.notebook.icon:
 		#	try:
 		#		self.set_icon_from_file(ui.notebook.icon)
-		#	except GObject.GError:
+		#	except (GObject.GError, GLib.Error):
 		#		logger.exception('Could not load icon %s', ui.notebook.icon)
 
 		page = notebook.get_page(page)
@@ -967,6 +989,12 @@ class PageWindow(Window):
 		self.add(self.pageview)
 
 
+		def do_delete_event(*a):
+			logger.debug('Close PageWindow for %s', page)
+			self.uistate['windowsize'] = tuple(self.get_size())
+
+		self.connect('delete-event', do_delete_event)
+
 class OpenPageDialog(Dialog):
 	'''Dialog to go to a specific page. Also known as the "Jump to" dialog.
 	Prompts for a page name and navigate to that page on 'Ok'.
